package cc.winboll.studio.libappbase.utils;

/**
 * @Author ZhanGSKen<zhangsken@188.com>
 * @Date 2025/06/04 13:36
 * @Describe RSA加密工具
 */
import cc.winboll.studio.libappbase.LogUtils;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import javax.crypto.Cipher;

public class RSAUtils {
    public static final String TAG = "RSAUtils";
    private static final RSAUtils INSTANCE = new RSAUtils();
    private static final int KEY_SIZE = 2048;
    private static final String KEY_ALGORITHM = "RSA";
    private static final String PUBLIC_KEY_FILE = "public.key";
    private static final String PRIVATE_KEY_FILE = "private.key";

    private RSAUtils() {}

    public static RSAUtils getInstance() {
        return INSTANCE;
    }

    /**
     * 检查密钥文件是否存在
     */
    public boolean keysExist(String path) {
        File publicKeyFile = new File(path + PUBLIC_KEY_FILE);
        File privateKeyFile = new File(path + PRIVATE_KEY_FILE);
        return publicKeyFile.exists() && privateKeyFile.exists();
    }

    /**
     * 生成密钥对并保存为字节文件
     */
    public void generateAndSaveKeys(String path) throws Exception {
        LogUtils.d(TAG, "正在生成密钥对");
        // 生成密钥对
        KeyPairGenerator generator = KeyPairGenerator.getInstance(KEY_ALGORITHM);
        generator.initialize(KEY_SIZE);
        KeyPair keyPair = generator.generateKeyPair();

        // 保存公钥（X.509字节）
        saveKey(path, PUBLIC_KEY_FILE, keyPair.getPublic().getEncoded());
        // 保存私钥（PKCS#8字节）
        saveKey(path, PRIVATE_KEY_FILE, keyPair.getPrivate().getEncoded());
    }

    /**
     * 读取密钥对（存在则读取，不存在则生成）
     */
    public KeyPair getOrGenerateKeys(String path) throws Exception {
        if (keysExist(path)) {
            LogUtils.d(TAG, "密钥对已存在");
            return readKeysFromFile(path);
        } else {
            LogUtils.d(TAG, "未生成密钥对");
            generateAndSaveKeys(path);
            return readKeysFromFile(path);
        }
    }

    /**
     * 从文件读取密钥对
     */
    private KeyPair readKeysFromFile(String path) throws Exception {
        LogUtils.d(TAG, "正在读取密钥对");
        byte[] publicKeyBytes = Files.readAllBytes(Paths.get(path + PUBLIC_KEY_FILE));
        byte[] privateKeyBytes = Files.readAllBytes(Paths.get(path + PRIVATE_KEY_FILE));

        X509EncodedKeySpec publicSpec = new X509EncodedKeySpec(publicKeyBytes);
        PKCS8EncodedKeySpec privateSpec = new PKCS8EncodedKeySpec(privateKeyBytes);

        KeyFactory factory = KeyFactory.getInstance(KEY_ALGORITHM);
        PublicKey publicKey = factory.generatePublic(publicSpec);
        PrivateKey privateKey = factory.generatePrivate(privateSpec);

        return new KeyPair(publicKey, privateKey);
    }

    /**
     * 通用保存密钥字节方法
     */
    private void saveKey(String path, String fileName, byte[] keyBytes) throws IOException {
        File dir = new File(path);
        if (!dir.exists()) dir.mkdirs(); // 自动创建目录
        try (FileOutputStream fos = new FileOutputStream(path + fileName)) {
            fos.write(keyBytes);
        }
    }
    
    /**
     * 公钥加密
     */
    public byte[] encryptWithPublicKey(String plainText, PublicKey publicKey) throws Exception {
        Cipher cipher = Cipher.getInstance(KEY_ALGORITHM + "/ECB/PKCS1Padding");
        cipher.init(Cipher.ENCRYPT_MODE, publicKey);
        return cipher.doFinal(plainText.getBytes("UTF-8"));
    }

    /**
     * 私钥解密
     */
    public String decryptWithPrivateKey(byte[] encryptedData, PrivateKey privateKey) throws Exception {
        Cipher cipher = Cipher.getInstance(KEY_ALGORITHM + "/ECB/PKCS1Padding");
        cipher.init(Cipher.DECRYPT_MODE, privateKey);
        byte[] decryptedBytes = cipher.doFinal(encryptedData);
        return new String(decryptedBytes, "UTF-8");
    }
}

